@@ -2,6 +2,8 @@ import colorsys
 import re
 import sys
 import webbrowser
+import xml.etree.cElementTree as ET
+# from io import BytesIO
 
 from gi.repository import Gtk, Gdk, GObject, Pango
 from gi.repository.GdkPixbuf import Pixbuf
@@ -55,17 +57,10 @@ def updateCombo(combo, data):
     combo.set_active(new_active)
 
 
-# int.bit_length() new in Python 2.7, so we have to use an equivalent
-def int_bit_length(i):
-    string = bin(i)  # binary representation:  bin(-37) --> '-0b100101'
-    string = string.lstrip('-0b')  # remove leading zeros and minus sign
-    return len(string)  # len('100101') --> 6
-
-
 def genColor(n, startpoint=0):
     assert n >= 1
     # This splits the 0 - 1 segment in the pizza way
-    hue = (2 * n - 1) / (2.**int_bit_length(n - 1)) - 1
+    hue = (2 * n - 1) / (2.**(n - 1).bit_length()) - 1
     hue = (hue + startpoint) % 1
     # We set saturation based on the amount of green, scaled to the interval
     # [0.6..0.8]. This ensures a consistent lightness over all colors.
@@ -453,6 +448,11 @@ def initTexviewLinks(textview, text):
         tv_iter = textview.get_iter_at_location(int(event.x), int(event.y))
         if not tv_iter:
             return
+
+        # https://gramps-project.org/bugs/view.php?id=9335
+        if not isinstance(tv_iter, Gtk.TextIter):
+            tv_iter = tv_iter[1]
+
         for tag, link, msg_type, s, e in tags:
             if tv_iter.has_tag(tag):
                 tag.props.foreground = "red"
@@ -462,6 +462,11 @@ def initTexviewLinks(textview, text):
         tv_iter = textview.get_iter_at_location(int(event.x), int(event.y))
         if not tv_iter:
             return
+
+        # https://gramps-project.org/bugs/view.php?id=9335
+        if not isinstance(tv_iter, Gtk.TextIter):
+            tv_iter = tv_iter[1]
+
         for tag, link, msg_type, s, e in tags:
             if tv_iter and tv_iter.has_tag(tag) and \
                     tag.props.foreground_Gdk.red == 0xffff:
@@ -479,6 +484,11 @@ def initTexviewLinks(textview, text):
         tv_iter = textview.get_iter_at_location(int(event.x), int(event.y))
         if not tv_iter:
             return
+
+        # https://gramps-project.org/bugs/view.php?id=9335
+        if not isinstance(tv_iter, Gtk.TextIter):
+            tv_iter = tv_iter[1]
+
         for tag, link, msg_type, s, e in tags:
             if tv_iter.has_tag(tag):
                 textview.get_window(Gtk.TextWindowType.TEXT).set_cursor(
@@ -501,27 +511,26 @@ class GladeWidgets:
         into the python __getitem__ version """
 
     def __init__(self, filename):
-        self.builder = Gtk.Builder()
-        if not no_gettext:
-            self.builder.set_translation_domain("pychess")
-        self.builder.add_from_file(addDataPrefix("glade/%s" % filename))
-
         # TODO: remove this when upstream fixes translations with Python3+Windows
         if PY3 and sys.platform == "win32" and not no_gettext:
-            for obj in self.builder.get_objects():
-                if (not isinstance(obj, Gtk.SeparatorMenuItem)) and hasattr(
-                        obj, "get_label"):
-                    label = obj.get_label()
-                    if label is not None:
-                        obj.set_label(_(label))
-                elif hasattr(obj, "get_title"):
-                    title = obj.get_title()
-                    if title is not None:
-                        obj.set_title(_(title))
-                if hasattr(obj, "get_tooltip_text"):
-                    text = obj.get_tooltip_text()
-                    if text is not None:
-                        obj.set_tooltip_text(_(text))
+            tree = ET.parse(addDataPrefix("glade/%s" % filename))
+            for node in tree.iter():
+                if 'translatable' in node.attrib:
+                    node.text = _(node.text)
+                if node.get('name') in ('pixbuf', 'logo'):
+                    node.text = addDataPrefix("glade/%s" % node.text)
+            # temp_file = BytesIO()
+            temp_file = addDataPrefix("glade/%s.temp" % filename)
+            tree.write(temp_file, encoding='utf-8', xml_declaration=True)
+            # xml_text = temp_file.getvalue().decode()
+            # self.builder = Gtk.Builder.new_from_string(xml_text, len(xml_text))
+            self.builder = Gtk.Builder()
+            self.builder.add_from_file(temp_file)
+        else:
+            self.builder = Gtk.Builder()
+            if not no_gettext:
+                self.builder.set_translation_domain("pychess")
+            self.builder.add_from_file(addDataPrefix("glade/%s" % filename))
 
     def __getitem__(self, key):
         return self.builder.get_object(key)
