@@ -203,12 +203,14 @@ class FICSConnection(Connection):
                 else:
                     print("guest", file=self.client)
                 got = self.client.read_until(
-                    "Press return", "If it is yours, type the password.",
-                    "guest connections have been prevented")
-                # got = 2
+                    "Press return",
+                    "If it is yours, type the password.",
+                    "guest connections have been prevented",
+                    "nobody from your site may login without an account.")
+                # got = 3
                 if got == 1:
                     raise LogOnException(REGISTERED % self.username)
-                elif got == 2:
+                elif got == 2 or got == 3:
                     raise LogOnException(PREVENTED)
                 print(file=self.client)
 
@@ -322,8 +324,13 @@ class FICSMainConnection(FICSConnection):
         self.stored_owner = self.username
         self.history_owner = self.username
         self.journal_owner = self.username
+        self.set_user_vars = False
 
     def close(self):
+        if isinstance(self.client, PredictionsTelnet) and self.set_user_vars:
+            self.client.run_command("set open 0")
+            self.client.run_command("set gin 0")
+            self.client.run_command("set availinfo 0")
         try:
             self.lvm.stop()
         except AttributeError:
@@ -368,6 +375,22 @@ class FICSMainConnection(FICSConnection):
         self.seeks.start()
         self.challenges.start()
 
+    def start_helper_manager(self, set_user_vars):
+        # if guest accounts disabled we will handle players in the main connection
+        if self.FatICS or self.USCN:
+            self.client.run_command("set pin 1")
+        else:
+            self.client.run_command("iset allresults 1")
+            # ivar pin: http://www.freechess.org/Help/HelpFiles/new_features.html
+            self.client.run_command("iset pin 1")
+
+        self.set_user_vars = set_user_vars
+        if self.set_user_vars:
+            self.client.run_command("set open 1")
+            self.client.run_command("set gin 1")
+            self.client.run_command("set availinfo 1")
+        self.hm = HelperManager(self, self)
+
         # disable setting iveriables from console
         self.client.run_command("iset lock 1")
 
@@ -382,6 +405,7 @@ class FICSHelperConnection(FICSConnection):
         self.main_conn.lounge_loaded.wait()
         # set open 1 is a requirement for availinfo notifications
         self.client.run_command("set open 1")
+        self.client.run_command("set seek 0")
         self.client.run_command("set shout 0")
         self.client.run_command("set cshout 0")
         self.client.run_command("set tell 0")
@@ -395,3 +419,6 @@ class FICSHelperConnection(FICSConnection):
             # ivar pin: http://www.freechess.org/Help/HelpFiles/new_features.html
             self.client.run_command("iset pin 1")
         self.hm = HelperManager(self, self.main_conn)
+
+        # disable setting iveriables from console
+        self.main_conn.client.run_command("iset lock 1")
