@@ -7,7 +7,6 @@ from pychess.compat import unicode
 from pychess.System.Log import log
 from pychess.System.idle_add import idle_add
 from pychess.Utils.IconLoader import load_icon
-from pychess.Utils.Rating import Rating
 from pychess.Utils.const import ADJOURNED, WHITE, BLACK, UNSUPPORTED
 
 from pychess.ic import RATING_TYPES, IC_STATUS_PLAYING, IC_STATUS_OFFLINE, IC_STATUS_UNKNOWN, \
@@ -17,22 +16,12 @@ from pychess.ic import RATING_TYPES, IC_STATUS_PLAYING, IC_STATUS_OFFLINE, IC_ST
     TYPE_ATOMIC, TYPE_LIGHTNING, TYPE_BUGHOUSE, TYPE_CRAZYHOUSE, TYPE_LOSERS, TYPE_SUICIDE, \
     TYPE_WILD, GAME_TYPES_BY_RATING_TYPE, TYPE_UNREGISTERED, TYPE_COMPUTER, TYPE_ADMINISTRATOR, \
     DEVIATION_NONE, DEVIATION_ESTIMATED, DEVIATION_PROVISIONAL, GAME_TYPES_BY_FICS_NAME, \
-    GAME_TYPES
+    GAME_TYPES, TYPE_GRAND_MASTER, TYPE_INTERNATIONAL_MASTER, TYPE_FIDE_MASTER, \
+    TYPE_WOMAN_GRAND_MASTER, TYPE_WOMAN_INTERNATIONAL_MASTER, TYPE_WOMAN_FIDE_MASTER
 
 
-class FICSRatings(dict):
-    def __init__(self, *args, **kwargs):
-        dict.__init__(self, *args, **kwargs)
-
-        for ratingtype in RATING_TYPES:
-            self[ratingtype] = Rating(ratingtype, 0)
-
-    def __setitem__(self, key, val):
-        if key not in RATING_TYPES:
-            raise TypeError("bad key: %s %s" % (repr(key), type(key)))
-        elif not isinstance(val, Rating):
-            raise TypeError("bad val: %s %s" % (repr(val), type(val)))
-        dict.__setitem__(self, key, val)
+class AlreadyExistException(Exception):
+    pass
 
 
 @idle_add
@@ -85,7 +74,16 @@ def get_player_tooltip_text(player, show_status=True):
     return text
 
 
+def player_id(name):
+    """ Two players are equal if the first 11 characters of their name match.
+        This is to facilitate matching players from output of commands like the 'games'
+        command which only return the first 11 characters of a player's name """
+    return name[0:11].lower()
+
+
 class FICSPlayer(GObject.GObject):
+    __gsignals__ = {'ratings_changed': (GObject.SignalFlags.RUN_FIRST, None, (int, object))}
+
     def __init__(self,
                  name,
                  online=False,
@@ -95,14 +93,15 @@ class FICSPlayer(GObject.GObject):
         assert isinstance(name, str), name
         assert isinstance(online, bool), online
         GObject.GObject.__init__(self)
+        self.player_id = player_id(name)
         self.name = name
         self.online = online
         self._status = status
         self.status = status
         self.game = None
         self.adjournment = False
-        self.keep_after_logout = False  # Whether to remove from Players after they logout
-        self.ratings = FICSRatings()
+        self.ratings = [0 for ratingtype in RATING_TYPES]
+        self.deviations = [DEVIATION_NONE for ratingtype in RATING_TYPES]
         if titles is None:
             self.titles = set()
         else:
@@ -144,7 +143,6 @@ class FICSPlayer(GObject.GObject):
     def set_status(self, status):
         self._previous_status = self._status
         self._status = status
-
     status = GObject.property(get_status, set_status)
 
     def restore_previous_status(self):
@@ -204,52 +202,49 @@ class FICSPlayer(GObject.GObject):
 
     @property
     def blitz(self):
-        return self.ratings[TYPE_BLITZ].elo
+        return self.ratings[TYPE_BLITZ]
 
     @property
     def standard(self):
-        return self.ratings[TYPE_STANDARD].elo
+        return self.ratings[TYPE_STANDARD]
 
     @property
     def lightning(self):
-        return self.ratings[TYPE_LIGHTNING].elo
+        return self.ratings[TYPE_LIGHTNING]
 
     @property
     def atomic(self):
-        return self.ratings[TYPE_ATOMIC].elo
+        return self.ratings[TYPE_ATOMIC]
 
     @property
     def bughouse(self):
-        return self.ratings[TYPE_BUGHOUSE].elo
+        return self.ratings[TYPE_BUGHOUSE]
 
     @property
     def crazyhouse(self):
-        return self.ratings[TYPE_CRAZYHOUSE].elo
+        return self.ratings[TYPE_CRAZYHOUSE]
 
     @property
     def losers(self):
-        return self.ratings[TYPE_LOSERS].elo
+        return self.ratings[TYPE_LOSERS]
 
     @property
     def suicide(self):
-        return self.ratings[TYPE_SUICIDE].elo
+        return self.ratings[TYPE_SUICIDE]
 
     @property
     def wild(self):
-        return self.ratings[TYPE_WILD].elo
-
-    def __hash__(self):
-        """ Two players are equal if the first 10 characters of their name match.
-            This is to facilitate matching players from output of commands like the 'games'
-            command which only return the first 10 characters of a player's name """
-        return hash(self.name[0:10].lower())
+        return self.ratings[TYPE_WILD]
 
     def __eq__(self, player):
-        if isinstance(self, type(player)) and hash(self) == hash(player):
+        if isinstance(self, type(player)) and self.player_id == player.player_id:
             return True
         else:
             return False
 
+    def __hash__(self):
+        return hash(self.player_id)
+
     def __repr__(self):
         rep = "name='%s'" % (self.name + self.display_titles())
         rep += ", id=%s" % (id(self))
@@ -267,7 +262,7 @@ class FICSPlayer(GObject.GObject):
             if rating_type in self.ratings:
                 rep += ", %s=%s" % \
                     (GAME_TYPES_BY_RATING_TYPE[rating_type].display_text,
-                     repr(self.ratings[rating_type].elo))
+                     repr(self.ratings[rating_type]))
         return "<FICSPlayer " + rep + ">"
 
     def isAvailableForGame(self):
@@ -293,6 +288,15 @@ class FICSPlayer(GObject.GObject):
     def isAdmin(self):
         return TYPE_ADMINISTRATOR in self.titles
 
+    def isTitled(self):
+        return (
+            TYPE_GRAND_MASTER in self.titles) or (
+            TYPE_INTERNATIONAL_MASTER in self.titles) or (
+            TYPE_FIDE_MASTER in self.titles) or (
+            TYPE_WOMAN_GRAND_MASTER in self.titles) or (
+            TYPE_WOMAN_INTERNATIONAL_MASTER in self.titles) or (
+            TYPE_WOMAN_FIDE_MASTER in self.titles)
+
     @classmethod
     def getIconByRating(cls, rating, size=16):
         assert isinstance(rating, int), "rating not an int: %s" % str(rating)
@@ -343,31 +347,20 @@ class FICSPlayer(GObject.GObject):
 
         return markup
 
-    def copy(self):
-        player = FICSPlayer(self.name,
-                            online=self.online,
-                            status=self.status,
-                            titles=self.titles.copy())
-        for ratingtype, rating in self.ratings.items():
-            player.ratings[ratingtype] = rating.copy()
-        player.game = self.game
-        player.adjournment = self.adjournment
-        return player
-
     def getRatingMean(self):
         ratingtotal = 0
         numratings = 0
-        for ratingtype in self.ratings:
-            if self.ratings[ratingtype].elo == 0:
+        for ratingtype in RATING_TYPES:
+            if self.ratings[ratingtype] == 0:
                 continue
-            if self.ratings[ratingtype].deviation == DEVIATION_NONE:
-                ratingtotal += self.ratings[ratingtype].elo * 3
+            if self.deviations[ratingtype] == DEVIATION_NONE:
+                ratingtotal += self.ratings[ratingtype] * 3
                 numratings += 3
-            elif self.ratings[ratingtype].deviation == DEVIATION_ESTIMATED:
-                ratingtotal += self.ratings[ratingtype].elo * 2
+            elif self.deviations[ratingtype] == DEVIATION_ESTIMATED:
+                ratingtotal += self.ratings[ratingtype] * 2
                 numratings += 2
-            elif self.ratings[ratingtype].deviation == DEVIATION_PROVISIONAL:
-                ratingtotal += self.ratings[ratingtype].elo * 1
+            elif self.deviations[ratingtype] == DEVIATION_PROVISIONAL:
+                ratingtotal += self.ratings[ratingtype] * 1
                 numratings += 1
         return numratings > 0 and ratingtotal // numratings or 0
 
@@ -375,21 +368,19 @@ class FICSPlayer(GObject.GObject):
     # and deflated lightning ratings and needs work
     # IDEA: use rank in addition to rating to determine strength
     def getStrength(self):
-        if TYPE_BLITZ in self.ratings and \
-                self.ratings[TYPE_BLITZ].deviation == DEVIATION_NONE:
-            return self.ratings[TYPE_BLITZ].elo
-        elif TYPE_LIGHTNING in self.ratings and \
-                self.ratings[TYPE_LIGHTNING].deviation == DEVIATION_NONE:
-            return self.ratings[TYPE_LIGHTNING].elo
+        if self.deviations[TYPE_BLITZ] == DEVIATION_NONE:
+            return self.ratings[TYPE_BLITZ]
+        elif self.deviations[TYPE_LIGHTNING] == DEVIATION_NONE:
+            return self.ratings[TYPE_LIGHTNING]
         else:
             return self.getRatingMean()
 
     def getRatingByGameType(self, game_type):
         try:
-            return self.ratings[game_type.rating_type].elo
+            return self.ratings[game_type.rating_type]
         except KeyError:
             return 0
-        except AttributeError:
+        except IndexError:
             return 0
 
     def getRatingForCurrentGame(self):
@@ -415,43 +406,42 @@ class FICSPlayers(GObject.GObject):
         #        self.connection.fm.connect("fingeringFinished", self.onFinger)
         pass
 
-    def __getitem__(self, player):
-        if not isinstance(player, FICSPlayer):
-            raise TypeError("%s" % repr(player))
-        if hash(player) in self.players:
-            return self.players[hash(player)]
+    def __getitem__(self, name):
+        if not isinstance(name, str):
+            raise TypeError("%s" % name)
+        if name in self.players:
+            return self.players[name]
         else:
             raise KeyError
 
     def __setitem__(self, key, value):
-        """ key and value must be the same FICSPlayer object """
-        if not isinstance(key, FICSPlayer):
+        """ key must be str and value must be FICSPlayer object """
+        if not isinstance(key, str):
             raise TypeError
         if not isinstance(value, FICSPlayer):
             raise TypeError
-        if key != value:
-            raise Exception("Not the same: %s %s" % (repr(key), repr(value)))
-        if hash(value) in self.players:
-            raise Exception("%s already exists in %s" %
-                            (repr(value), repr(self)))
-        self.players[hash(value)] = value
-        self.players_cids[hash(value)] = value.connect("notify::online",
-                                                       self.online_changed)
+        if value.player_id in self.players:
+            raise AlreadyExistException
 
-    def __delitem__(self, player):
-        if not isinstance(player, FICSPlayer):
-            raise TypeError
-        if player in self:
-            del self.players[hash(player)]
-        if hash(player) in self.players_cids:
-            if player.handler_is_connected(self.players_cids[hash(player)]):
-                player.disconnect(self.players_cids[hash(player)])
-            del self.players_cids[hash(player)]
+        self.players[value.player_id] = value
+        self.players_cids[value.player_id] = value.connect(
+            "notify::online", self.online_changed)
 
-    def __contains__(self, player):
-        if not isinstance(player, FICSPlayer):
+    def __delitem__(self, name):
+        if not isinstance(name, str):
             raise TypeError
-        if hash(player) in self.players:
+        if name in self.players:
+            player = self.players[name]
+            if name in self.players_cids:
+                if player.handler_is_connected(self.players_cids[name]):
+                    player.disconnect(self.players_cids[name])
+                del self.players_cids[name]
+            del self.players[name]
+
+    def __contains__(self, name):
+        if not isinstance(name, str):
+            raise TypeError
+        if name in self.players:
             return True
         else:
             return False
@@ -482,25 +472,28 @@ class FICSPlayers(GObject.GObject):
                 names.append(player.name)
         return names
 
-    def get(self, player, create=True):
-        if player in self:
-            player = self[player]
-        elif create:
-            self[player] = player
+    def get(self, name):
+        key = player_id(name)
+        if key in self:
+            player = self[key]
         else:
-            raise KeyError
+            player = FICSPlayer(name)
+            try:
+                self[key] = player
+            except AlreadyExistException:
+                player = self[key]
         return player
 
-    def player_disconnected(self, player):
+    def player_disconnected(self, name):
         # log.debug("%s" % player,
         #    extra={"task": (self.connection.username, "player_disconnected")})
-        if player in self:
-            player = self[player]
+        key = player_id(name)
+        if key in self:
+            player = self[key]
             player.online = False
             player.status = IC_STATUS_OFFLINE
-            if not player.adjournment and not player.keep_after_logout and \
-                    player.name not in self.connection.notify_users:
-                del self[player]
+            if not player.adjournment and player.name not in self.connection.notify_users:
+                del self[player.player_id]
             else:
                 log.debug("Not removing %s" % player,
                           extra={"task": (self.connection.username,
@@ -509,9 +502,12 @@ class FICSPlayers(GObject.GObject):
                           'FICSPlayerExited',
                           player,
                           priority=GLib.PRIORITY_LOW)
-
+        else:
+            log.debug("Not removing %s (it's not in FICSPlayers)" % name,
+                      extra={"task": (self.connection.username,
+                                      "player_disconnected")})
     #    def onFinger (self, fm, finger):
-    #        player = FICSPlayer(finger.getName())
+    #        player = player_id(finger.getName())
     #        if player in self:
     #            self[player].finger = finger
     #            # TODO: merge ratings and titles from finger object into ficsplayer object
@@ -868,8 +864,8 @@ class FICSGame(FICSMatch):
         self.relation = relation
 
     def __hash__(self):
-        return hash(":".join((self.wplayer.name[0:10].lower(
-        ), self.bplayer.name[0:10].lower(), str(self.gameno))))
+        return hash(":".join((self.wplayer.name[0:11].lower(
+        ), self.bplayer.name[0:11].lower(), str(self.gameno))))
 
     def __eq__(self, game):
         if isinstance(game, FICSGame) and hash(self) == hash(game):
@@ -1046,8 +1042,8 @@ class FICSHistoryGame(FICSGame):
         self.history_no = history_no
 
     def __hash__(self):
-        return hash(":".join((self.wplayer.name[0:10].lower(
-        ), self.bplayer.name[0:10].lower(), str(self.history_no), str(
+        return hash(":".join((self.wplayer.name[0:11].lower(
+        ), self.bplayer.name[0:11].lower(), str(self.history_no), str(
             self.time))))
 
     @property
@@ -1095,8 +1091,8 @@ class FICSJournalGame(FICSGame):
         self.journal_no = journal_no
 
     def __hash__(self):
-        return hash(":".join((self.wplayer.name[0:10].lower(),
-                              self.bplayer.name[0:10].lower(),
+        return hash(":".join((self.wplayer.name[0:11].lower(),
+                              self.bplayer.name[0:11].lower(),
                               str(self.journal_no), )))
 
     @property
