@@ -260,8 +260,9 @@ class PredictionsTelnet(object):
         if not line.line:
             return  # TODO: necessary?
 
-        for pred in self.reply_cmd_dict[line.code] \
-                if line.code and line.code in self.reply_cmd_dict else self.predictions:
+        predictions = self.reply_cmd_dict[line.code] \
+            if line.code and line.code in self.reply_cmd_dict else self.predictions
+        for pred in list(predictions):
             #            print "parse_line: trying prediction %s for line '%s'" % (pred.name, line)
             answer = self.test_prediction(pred, line)
             if answer in (RETURN_MATCH, RETURN_MATCH_END):
@@ -306,5 +307,7 @@ class PredictionsTelnet(object):
         self.telnet.cancel()
 
     def close(self):
+        # save played game (if there is any) if no moves made
+        self.run_command("abort")
         self.run_command("quit")
         self.telnet.close()
