@@ -126,15 +126,13 @@ class WildGameType(VariantGameType):
 GAME_TYPES = {
     "blitz": NormalGameType("blitz", "b", TYPE_BLITZ, _("Blitz")),
     "standard": NormalGameType("standard", "s", TYPE_STANDARD, _("Standard")),
-    "lightning":
-    NormalGameType("lightning", "l", TYPE_LIGHTNING, _("Lightning")),
+    "lightning": NormalGameType("lightning", "l", TYPE_LIGHTNING, _("Lightning")),
     "untimed": NormalGameType("untimed", "u", TYPE_UNTIMED, _("Untimed")),
     "examined": NormalGameType("examined", "e", TYPE_EXAMINED, _("Examined")),
     "nonstandard": NormalGameType("nonstandard", "n", TYPE_OTHER, _("Other")),
     "atomic": VariantGameType("atomic", "x", TYPE_ATOMIC, ATOMICCHESS),
     "bughouse": VariantGameType("bughouse", "B", TYPE_BUGHOUSE, BUGHOUSECHESS),
-    "crazyhouse":
-    VariantGameType("crazyhouse", "z", TYPE_CRAZYHOUSE, CRAZYHOUSECHESS),
+    "crazyhouse": VariantGameType("crazyhouse", "z", TYPE_CRAZYHOUSE, CRAZYHOUSECHESS),
     "losers": VariantGameType("losers", "L", TYPE_LOSERS, LOSERSCHESS),
     "suicide": VariantGameType("suicide", "S", TYPE_SUICIDE, SUICIDECHESS),
     "wild/fr": WildGameType("wild/fr", FISCHERRANDOMCHESS),
@@ -279,6 +277,14 @@ def parse_title_hex(titlehex):
     return titles
 
 
+def parseRating(rating):
+    if rating[0] == " ":
+        rating = rating[1:]
+    if rating[-1].isalpha():
+        rating = rating[:-1]
+    return int(rating) if rating.isdigit() else 0
+
+
 def get_infobarmessage_content(player, text, gametype=None):
     content = Gtk.HBox()
     icon = Gtk.Image()
@@ -527,3 +533,92 @@ BLKCMD_ERROR_NOSEQUENCE = 519
 BLKCMD_ERROR_LENGTH = 520
 
 LIMIT_BLKCMD_ERRORS = 500
+
+FICS_COMMANDS = [
+    'abort', 'accept', 'addlist', 'adjourn', 'alias', 'allobservers', 'assess',
+    'backward', 'bell', 'best', 'boards', 'bsetup', 'bugwho', 'cbest',
+    'clearmessages', 'convert_bcf', 'convert_elo', 'convert_uscf', 'copygame',
+    'crank', 'cshout', 'date', 'decline', 'draw', 'examine', 'finger', 'flag',
+    'flip', 'fmessage', 'follow', 'forward', 'games', 'gnotify', 'goboard',
+    'handles', 'hbest', 'help', 'history', 'hrank', 'inchannel', 'index',
+    'info', 'it', 'jkill', 'jsave', 'kibitz', 'limits', 'llogons', 'logons',
+    'mailhelp', 'mailmess', 'mailmoves', 'mailoldmoves', 'mailsource',
+    'mailstored', 'match', 'messages', 'mexamine', 'moretime', 'moves', 'news',
+    'next', 'observe', 'oldmoves', 'open', 'password', 'pause', 'pending',
+    'pfollow', 'play', 'pobserve', 'promote', 'pstat', 'qtell', 'quit', 'rank',
+    'refresh', 'resign', 'resume', 'revert', 'say', 'seek', 'servers', 'set',
+    'shout', 'showlist', 'simabort', 'simallabort', 'simadjourn',
+    'simalladjourn', 'simgames', 'simmatch', 'simnext', 'simobserve',
+    'simopen', 'simpass', 'simprev', 'smoves', 'smposition', 'sought',
+    'sposition', 'statistics', 'stored', 'style', 'sublist', 'switch',
+    'takeback', 'tell', 'time', 'unalias', 'unexamine', 'unobserve', 'unpause',
+    'unseek', 'uptime', 'ustat', 'variables', 'whisper', 'who', 'withdraw',
+    'xkibitz', 'xtell', 'xwhisper', 'znotify']
+
+FICS_HELP = [
+    '_index', 'abort', 'abuse', 'academy', 'accept', 'addlist', 'addresses',
+    'adjourn', 'adjournments', 'adjudicate', 'adjudication', 'adm_app',
+    'adm_info', 'adm_new', 'admins', 'alias', 'allobservers', 'assess',
+    'atomic', 'audiochat', 'avail_vars', 'backward', 'bclock', 'bell', 'best',
+    'blind', 'blindfold', 'blindh', 'blitz', 'block_codes', 'bname', 'boards',
+    'brating', 'bsetup', 'bughouse', 'bughouse_strat', 'bugreport', 'bugwho',
+    'busy', 'ca', 'category', 'cbest', 'censor', 'chan_1', 'chan_4', 'channel',
+    'channel_list', 'channels', 'chess_adviser', 'chess_advisor',
+    'clearmessage', 'clearmessages', 'clock', 'clocks', 'clrsquare', 'cls',
+    'cls_info', 'command', 'commands', 'commit', 'computer_app',
+    'computer_list', 'computers', 'confidentiality', 'convert_bcf',
+    'convert_elo', 'convert_uscf', 'copygame', 'crank', 'crazyhouse',
+    'crazyhouse_strat', 'credit', 'crstat', 'cshout', 'csnewse', 'csnewsf',
+    'csnewsi', 'csnewsp', 'csnewst', 'date', 'decline', 'disclaimer',
+    'disconnection', 'draw', 'eco', 'eggo', 'email', 'etime', 'examine', 'exl',
+    'fen', 'fics_faq', 'fics_lingo', 'finger', 'flag', 'flip', 'fmessage',
+    'follow', 'formula', 'forward', 'fr', 'fr_rules', 'ftp_hints', 'games',
+    'games', 'getgame', 'getgi', 'getpi', 'ginfo', 'glicko', 'gnotify',
+    'goboard', 'handle', 'handles', 'hbest', 'help', 'highlight', 'history',
+    'hrank', 'hrstat', 'hstat', 'icsdrone', 'idlenotify', 'inchannel', 'index',
+    'indexfile', 'inetchesslib', 'info', 'intellegence', 'interfaces',
+    'intro_analysis', 'intro_basics', 'intro_general', 'intro_information',
+    'intro_moving', 'intro_playing', 'intro_settings', 'intro_talking',
+    'intro_welcome', 'irc_help', 'iset', 'it', 'iv_allresults', 'iv_atomic',
+    'iv_audiochat', 'iv_block', 'iv_boardinfo', 'iv_compressmove',
+    'iv_crazyhouse', 'iv_defprompt', 'iv_extascii', 'iv_extuserinfo', 'iv_fr',
+    'iv_gameinfo', 'iv_graph', 'iv_list', 'iv_lock', 'iv_pendinfo',
+    'iv_seekinfo', 'iv_seekremove', 'iv_startpos', 'ivariables', 'jkill',
+    'journal', 'jsave', 'kibitz', 'kiblevel', 'lag', 'lecture1', 'lessons',
+    'lightning', 'limits', 'links', 'lists', 'llogons', 'logons', 'losers',
+    'losers_chess', 'mailhelp', 'mailmess', 'mailmoves', 'mailoldmoves',
+    'mailstored', 'mamer', 'manual_usage', 'manual_vars', 'match',
+    'meeting_1_followup', 'meeting_1_long', 'meeting_1_short',
+    'meetings_index', 'messages', 'mexamine', 'moretime', 'motd', 'motd_fri',
+    'motd_help', 'motd_mon', 'motd_sat', 'motd_sun', 'motd_thu', 'motd_tue',
+    'motd_wed', 'moves', 'mule', 'new_features', 'newbie', 'news', 'next',
+    'noescape', 'noplay', 'notes', 'notify', 'observe', 'odds', 'oldmoves',
+    'oldpstat', 'open', 'partner', 'password', 'pause', 'pending', 'pfollow',
+    'pgn', 'ping', 'play', 'pobserve', 'powericsfaq', 'prefresh', 'primary',
+    'private', 'promote', 'pstat', 'ptell', 'ptime', 'qtell', 'quit', 'rank',
+    'rating_changes', 'ratings', 'rcopygame', 'rd', 'refresh', 'register',
+    'relay', 'relay_operator', 'rematch', 'replay', 'resign', 'result',
+    'resume', 'revert', 'rfollow', 'rmatch', 'robofics', 'robserve', 'rstat',
+    'sabort', 'say', 'sdraw', 'seek', 'servers', 'set', 'setup', 'shout',
+    'shout_quota', 'showadmins', 'showlist', 'showsrs', 'simabort',
+    'simadjourn', 'simallabort', 'simalladjourn', 'simgames', 'simmatch',
+    'simnext', 'simobserve', 'simopen', 'simpass', 'simprev', 'simuls',
+    'skype', 'smoves', 'smposition', 'sought', 'spending', 'sposition', 'sr',
+    'sr_info', 'standard', 'statistics', 'stats', 'stc', 'stored', 'style',
+    'style12', 'sublist', 'suicide_chess', 'summon', 'switch', 'system_alias',
+    'takeback', 'team', 'teamgames', 'tell', 'time', 'timeseal',
+    'timeseal_mac', 'timeseal_os2', 'timeseal_unix', 'timeseal_windows',
+    'timezones', 'tm', 'tomove', 'totals', 'totals_info', 'tournset',
+    'town_meetings', 'townmtg1', 'unalias', 'unexamine', 'unobserve',
+    'unpause', 'unseek', 'untimed', 'uptime', 'uscf', 'uscf_faq', 'ustat',
+    'v_autoflag', 'v_automail', 'v_availinfo', 'v_availmax', 'v_availmin',
+    'v_bell', 'v_bugopen', 'v_chanoff', 'v_cshout', 'v_ctell', 'v_echo',
+    'v_flip', 'v_formula', 'v_gin', 'v_height', 'v_highlight', 'v_inc',
+    'v_interface', 'v_jprivate', 'v_kibitz', 'v_kiblevel', 'v_language',
+    'v_mailmess', 'v_messreply', 'v_notakeback', 'v_notifiedby', 'v_open',
+    'v_pgn', 'v_pin', 'v_private', 'v_prompt', 'v_provshow', 'v_ptime',
+    'v_rated', 'v_ropen', 'v_seek', 'v_shout', 'v_silence', 'v_simopen',
+    'v_style', 'v_tell', 'v_time', 'v_tolerance', 'v_tourney', 'v_tzone',
+    'v_unobserve', 'v_width', 'variables', 'wclock', 'webpage', 'whenshut',
+    'whisper', 'who', 'wild', 'withdraw', 'wname', 'wrating', 'xkibitz',
+    'xtell', 'xwhisper', 'zhouse', 'znotify']
