@@ -23,9 +23,9 @@ from pychess.Utils.const import WHITEWON, WON_RESIGN, WON_DISCONNECTION, WON_CAL
 from pychess.ic import IC_POS_INITIAL, IC_POS_ISOLATED, IC_POS_OP_TO_MOVE, IC_POS_ME_TO_MOVE, \
     IC_POS_OBSERVING, IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, GAME_TYPES, IC_STATUS_PLAYING, \
     BLKCMD_SEEK, BLKCMD_OBSERVE, BLKCMD_MATCH, TYPE_WILD, BLKCMD_SMOVES, BLKCMD_UNOBSERVE, BLKCMD_MOVES, \
-    BLKCMD_FLAG
+    BLKCMD_FLAG, parseRating
 
-from pychess.ic.FICSObjects import FICSPlayer, FICSGame, FICSBoard, FICSHistoryGame, \
+from pychess.ic.FICSObjects import FICSGame, FICSBoard, FICSHistoryGame, \
     FICSAdjournedGame, FICSJournalGame
 
 names = "(\w+)"
@@ -51,7 +51,7 @@ moveListHeader2Str = "%s ([^ ]+) match, initial time: (\d+) minutes, increment:
     ratedexp
 moveListHeader2 = re.compile(moveListHeader2Str, re.IGNORECASE)
 sanmove = "([a-hx@OoPKQRBN0-8+#=-]{2,7})"
-movetime = "\((\d:)?(\d{1,2}):(\d\d)(?:\.(\d\d\d))?\)"
+movetime = "\((\d:)?(\d{1,2}):(\d\d)(?:\.(\d{1,3}))?\)"
 moveListMoves = re.compile("\s*(\d+)\. +(?:%s|\.\.\.) +%s *(?:%s +%s)?" %
                            (sanmove, movetime, sanmove, movetime))
 
@@ -193,6 +193,8 @@ class BoardManager(GObject.GObject):
                                (object, )),
         'boardUpdate': (GObject.SignalFlags.RUN_FIRST, None,
                         (int, int, int, str, str, str, str, int, int)),
+        'timesUpdate': (GObject.SignalFlags.RUN_FIRST, None,
+                        (int, int, int,)),
         'obsGameEnded': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'curGameEnded': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'obsGameUnobserved': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
@@ -428,8 +430,8 @@ class BoardManager(GObject.GObject):
                 ("SetUp", "1"), ("FEN", fen)
             ]
             pgn = "\n".join(['[%s "%s"]' % line for line in pgnHead]) + "\n*\n"
-            wplayer = self.connection.players.get(FICSPlayer(wname))
-            bplayer = self.connection.players.get(FICSPlayer(bname))
+            wplayer = self.connection.players.get(wname)
+            bplayer = self.connection.players.get(bname)
 
             # examine from console or got mexamine in observed game
             if self.connection.examined_game is None:
@@ -463,7 +465,7 @@ class BoardManager(GObject.GObject):
 
             game.relation = relation
             game.board = FICSBoard(0, 0, pgn=pgn)
-            self.gamesImObserving[game] = None
+            self.gamesImObserving[game] = wms, bms
 
             # start a new game now or after smoves
             self.gamemodelStartedEvents[game.gameno] = threading.Event()
@@ -512,9 +514,7 @@ class BoardManager(GObject.GObject):
 
     def matchDeclined(self, match):
         decliner, = match.groups()
-        decliner = self.connection.players.get(
-            FICSPlayer(decliner),
-            create=False)
+        decliner = self.connection.players.get(decliner)
         self.emit("matchDeclined", decliner)
 
     @classmethod
@@ -527,12 +527,6 @@ class BoardManager(GObject.GObject):
         else:
             return ("k", "q")
 
-    @staticmethod
-    def parseRating(rating):
-        if rating[-1].isalpha():
-            rating = rating[:-1]
-        return int(rating) if rating.isdigit() else 0
-
     def onPlayGameCreated(self, matchlist):
         log.debug(
             "'%s' '%s' '%s'" %
@@ -543,17 +537,18 @@ class BoardManager(GObject.GObject):
         item = 2 if self.connection.USCN else 1
         gameno, wname, bname, rated, match_type = matchlist[item].groups()
         gameno = int(gameno)
-        wrating = self.parseRating(wrating)
-        brating = self.parseRating(brating)
+        wrating = parseRating(wrating)
+        brating = parseRating(brating)
         rated = rated == "rated"
         game_type = GAME_TYPES[match_type]
 
-        wplayer = self.connection.players.get(FICSPlayer(wname))
-        bplayer = self.connection.players.get(FICSPlayer(bname))
+        wplayer = self.connection.players.get(wname)
+        bplayer = self.connection.players.get(bname)
         for player, rating in ((wplayer, wrating), (bplayer, brating)):
             if game_type.rating_type in player.ratings and \
-                    player.ratings[game_type.rating_type].elo != rating:
-                player.ratings[game_type.rating_type].elo = rating
+                    player.ratings[game_type.rating_type] != rating:
+                player.ratings[game_type.rating_type] = rating
+                player.emit("ratings_changed", game_type.rating_type, player)
 
         style12 = matchlist[-1].groups()[0]
         castleSigns = self.generateCastleSigns(style12, game_type)
@@ -579,7 +574,6 @@ class BoardManager(GObject.GObject):
                 player.status = IC_STATUS_PLAYING
             if player.game != game:
                 player.game = game
-            player.keep_after_logout = True
 
         self.theGameImPlaying = game
         self.gamemodelStartedEvents[gameno] = threading.Event()
@@ -719,8 +713,8 @@ class BoardManager(GObject.GObject):
             weekday, month, day, hour, minute, timezone, year = matches[4:11]
             month = months.index(month) + 1
 
-        wrating = self.parseRating(wrating)
-        brating = self.parseRating(brating)
+        wrating = parseRating(wrating)
+        brating = parseRating(brating)
         rated, game_type, minutes, increment = \
             moveListHeader2.match(matchlist[index + 1]).groups()
         minutes = int(minutes)
@@ -769,6 +763,7 @@ class BoardManager(GObject.GObject):
         moves = {}
         times = {}
         wms = bms = minutes * 60 * 1000
+
         for line in matchlist[movesstart:-1]:
             if not moveListMoves.match(line):
                 log.error("BoardManager.parseGame: unmatched line: \"%s\"" %
@@ -803,6 +798,7 @@ class BoardManager(GObject.GObject):
                     bms += (increment * 1000)
                 times[ply + 1] = "%01d:%02d:%02d.%03d" % (
                     int(bhour), int(bmin), int(bsec), int(bmsec))
+
         if in_progress and gameno in self.queuedStyle12s:
             # Apply queued board updates
             for style12 in self.queuedStyle12s[gameno]:
@@ -865,13 +861,13 @@ class BoardManager(GObject.GObject):
             pgn += "%s {[%%emt %s]} " % (move, time)
         pgn += "*\n"
 
-        wplayer = self.connection.players.get(FICSPlayer(wname))
-        bplayer = self.connection.players.get(FICSPlayer(bname))
+        wplayer = self.connection.players.get(wname)
+        bplayer = self.connection.players.get(bname)
         for player, rating in ((wplayer, wrating), (bplayer, brating)):
             if game_type.rating_type in player.ratings and \
-                    player.ratings[game_type.rating_type].elo != rating:
-                player.ratings[game_type.rating_type].elo = rating
-            player.keep_after_logout = True
+                    player.ratings[game_type.rating_type] != rating:
+                player.ratings[game_type.rating_type] = rating
+                player.emit("ratings_changed", game_type.rating_type, player)
         game = gameclass(wplayer,
                          bplayer,
                          game_type=game_type,
@@ -904,8 +900,8 @@ class BoardManager(GObject.GObject):
         else:
             gameno, wname, wrating, bname, brating, rated, gametype, minutes, inc = matchlist[
                 1].groups()
-            wrating = self.parseRating(wrating)
-            brating = self.parseRating(brating)
+            wrating = parseRating(wrating)
+            brating = parseRating(brating)
             game_type = GAME_TYPES[gametype]
 
         style12 = matchlist[-1].groups()[0]
@@ -916,8 +912,8 @@ class BoardManager(GObject.GObject):
         gameno = int(gameno)
         self.castleSigns[gameno] = castleSigns
 
-        wplayer = self.connection.players.get(FICSPlayer(wname))
-        bplayer = self.connection.players.get(FICSPlayer(bname))
+        wplayer = self.connection.players.get(wname)
+        bplayer = self.connection.players.get(bname)
 
         if relation == IC_POS_OBSERVING_EXAMINATION:
             pgnHead = [
@@ -938,7 +934,7 @@ class BoardManager(GObject.GObject):
                                             pgn=pgn),
                             relation=relation)
             game = self.connection.games.get(game)
-            self.gamesImObserving[game] = None
+            self.gamesImObserving[game] = wms, bms
 
             self.gamemodelStartedEvents[game.gameno] = threading.Event()
             self.emit("obsGameCreated", game)
@@ -964,7 +960,7 @@ class BoardManager(GObject.GObject):
                             game.gameno)
                 return
 
-            self.gamesImObserving[game] = None
+            self.gamesImObserving[game] = wms, bms
             self.queuedStyle12s[game.gameno] = []
             self.queuedEmits[game.gameno] = []
             self.gamemodelStartedEvents[game.gameno] = threading.Event()
@@ -981,15 +977,22 @@ class BoardManager(GObject.GObject):
         game = self.parseGame(matchlist, FICSGame, in_progress=True)
         if game.gameno not in self.gamemodelStartedEvents:
             return
+        if game.gameno not in self.queuedEmits:
+            return
+
         self.emit("obsGameCreated", game)
         try:
             self.gamemodelStartedEvents[game.gameno].wait()
         except KeyError:
             pass
+
         for emit in self.queuedEmits[game.gameno]:
             emit()
         del self.queuedEmits[game.gameno]
 
+        wms, bms = self.gamesImObserving[game]
+        self.emit("timesUpdate", game.gameno, wms, bms)
+
     onObserveGameMovesReceived.BLKCMD = BLKCMD_MOVES
 
     def onArchiveGameSMovesReceived(self, matchlist):
@@ -1071,7 +1074,7 @@ class BoardManager(GObject.GObject):
 
     def player_lagged(self, match):
         gameno, player, num_seconds = match.groups()
-        player = self.connection.players.get(FICSPlayer(player))
+        player = self.connection.players.get(player)
         self.emit("player_lagged", player)
 
     def opp_not_out_of_time(self, match):
@@ -1081,21 +1084,21 @@ class BoardManager(GObject.GObject):
 
     def req_not_fit_formula(self, matchlist):
         player, formula = matchlist[1].groups()
-        player = self.connection.players.get(FICSPlayer(player))
+        player = self.connection.players.get(player)
         self.emit("req_not_fit_formula", player, formula)
 
     req_not_fit_formula.BLKCMD = BLKCMD_MATCH
 
     def player_on_censor(self, match):
         player, = match.groups()
-        player = self.connection.players.get(FICSPlayer(player))
+        player = self.connection.players.get(player)
         self.emit("player_on_censor", player)
 
     player_on_censor.BLKCMD = BLKCMD_MATCH
 
     def player_on_noplay(self, match):
         player, = match.groups()
-        player = self.connection.players.get(FICSPlayer(player))
+        player = self.connection.players.get(player)
         self.emit("player_on_noplay", player)
 
     player_on_noplay.BLKCMD = BLKCMD_MATCH
@@ -1143,6 +1146,9 @@ class BoardManager(GObject.GObject):
         elif player is not None:
             self.connection.client.run_command("observe %s" % player.name)
 
+    def follow(self, player):
+        self.connection.client.run_command("follow %s" % player.name)
+
     def unexamine(self):
         self.connection.client.run_command("unexamine")
 
