@@ -6,7 +6,6 @@ from gi.repository import GObject
 from pychess.ic import IC_STATUS_OFFLINE, IC_STATUS_ACTIVE, IC_STATUS_PLAYING, IC_STATUS_BUSY, \
     GAME_TYPES_BY_FICS_NAME, BLKCMD_FINGER
 from pychess.Utils.const import WHITE, BLACK
-from pychess.Utils.Rating import Rating
 from pychess.System.Log import log
 
 types = "(?:blitz|standard|lightning|wild|bughouse|crazyhouse|suicide|losers|atomic)"
@@ -48,7 +47,7 @@ class FingerObject:
         self.__silence = False
         self.__titles = None
 
-        self.__rating = {}
+        self.__ratings = {}
 
     def getName(self):
         """ Returns the name of the user, without any title sufixes """
@@ -84,7 +83,7 @@ class FingerObject:
         """ Returns how many seconds the user has been on FICS since the account
             was created.
             This is not set, if the user has never logged on """
-        return self.__totalTimeOnline + time() - self.__fingerTime
+        return self.__totalTimeOnline
 
     def getCreated(self):
         """ Returns when the account was created """
@@ -134,10 +133,14 @@ class FingerObject:
             This is only set when status == STATUS_PLAYING """
         return self.__silence
 
+    def getRatings(self):
+        return self.__ratings
+
     def getRating(self, type=None):
-        if type is None:
-            return self.__rating
-        return self.__rating[type]
+        return int(self.__ratings[type][0])
+
+    def getRatingsLen(self):
+        return len(self.__ratings)
 
     def getTitles(self):
         return self.__titles
@@ -199,8 +202,8 @@ class FingerObject:
     def setSilence(self, value):
         self.__silence = value
 
-    def setRating(self, rating_type, rating):
-        self.__rating[rating_type] = rating
+    def setRating(self, rating_type, rating_line):
+        self.__ratings[rating_type] = rating_line
 
     def setTitles(self, titles):
         self.__titles = titles
@@ -249,15 +252,15 @@ class FingerManager(GObject.GObject):
 
     def parseDate(self, date):
         # Tue Mar 11, 10:56 PDT 2008
-        return 1
+        return date
 
     def parseShortDate(self, date):
         # 24-Oct-2007
-        return 1
+        return date
 
     def parseTime(self, time):
         # 3 days, 2 hrs, 53 mins
-        return 1
+        return time
 
     def onFinger(self, matchlist):
         finger = FingerObject()
@@ -298,16 +301,7 @@ class FingerManager(GObject.GObject):
                 gametype = GAME_TYPES_BY_FICS_NAME[groupdict["gametype"].lower(
                 )]
                 ratings = groupdict["ratings"].split()
-                del ratings[5]  # We don't need the totals field
-                ratings[1] = float(ratings[1])
-                if len(ratings) == 5:
-                    args = map(int, ratings)
-                    rating = Rating(gametype.rating_type, *args)
-                else:
-                    bestTime = self.parseShortDate(ratings[6][1:-1])
-                    args = list(map(int, ratings[:6])) + [bestTime]
-                    rating = Rating(gametype.rating_type, *args)
-                finger.setRating(gametype.rating_type, rating)
+                finger.setRating(gametype.rating_type, ratings)
             elif groupdict["email"] is not None:
                 finger.setEmail(groupdict["email"])
             elif groupdict["sanctions"] is not None:
@@ -315,7 +309,7 @@ class FingerManager(GObject.GObject):
             elif groupdict["tto"] is not None:
                 finger.setTotalTimeOnline(self.parseTime(groupdict["tto"]))
             elif groupdict["created"] is not None:
-                finger.setTotalTimeOnline(self.parseDate(groupdict["created"]))
+                finger.setCreated(self.parseDate(groupdict["created"]))
             elif groupdict["timeseal"] is not None:
                 finger.setTimeseal(groupdict["timeseal"] == "On")
             elif groupdict["adminlevel"] is not None:
