@@ -1,4 +1,5 @@
 import re
+from threading import Timer
 
 from gi.repository import GObject
 
@@ -7,7 +8,7 @@ from pychess.ic import GAME_TYPES_BY_SHORT_FICS_NAME, IC_STATUS_PLAYING, BLKCMD_
     GAME_TYPES, TITLES, TYPE_BLITZ, parse_title_hex, TYPE_ATOMIC, TYPE_WILD, \
     TYPE_STANDARD, TYPE_LIGHTNING, TYPE_CRAZYHOUSE, TYPE_LOSERS, TYPE_BUGHOUSE, \
     TYPE_SUICIDE, DEVIATION, STATUS, BLKCMD_WHO, IC_STATUS_NOT_AVAILABLE, \
-    IC_STATUS_AVAILABLE
+    IC_STATUS_AVAILABLE, parseRating
 from pychess.ic.FICSObjects import FICSPlayer, FICSGame
 from pychess.ic.managers.BoardManager import parse_reason
 
@@ -21,6 +22,8 @@ names = "([a-zA-Z]+)%s" % titles
 mf = "(?:([mf]{1,2})\s?)?"
 whomatch = "(?:(?:([-0-9+]{1,4})([\^~:\#. &])%s))" % names
 whomatch_re = re.compile(whomatch)
+whoI = "([A-Za-z]+)([\^~:\#. &])(\\d{2})" + "(\d{1,4})([P E])" * 8 + "(\d{1,4})([PE]?)"
+re_whoI = re.compile(whoI)
 
 
 class HelperManager(GObject.GObject):
@@ -30,13 +33,13 @@ class HelperManager(GObject.GObject):
         self.helperconn = helperconn
         self.connection = connection
 
-        self.helperconn.expect_line(
+        # TODO: Examined games
+        self.helperconn.expect_fromto(
             self.on_game_list,
             "(\d+) %s (\w+)\s+%s (\w+)\s+\[(p| )(%s)(u|r)\s*(\d+)\s+(\d+)\]\s*(\d:)?(\d+):(\d+)\s*-\s*(\d:)?(\d+):(\d+) \(\s*(\d+)-\s*(\d+)\) (W|B):\s*(\d+)"
             %
-            (ratings, ratings, "|".join(GAME_TYPES_BY_SHORT_FICS_NAME.keys())))
-        self.helperconn.expect_line(self.on_game_list_end,
-                                    "(\d+) games displayed .*")
+            (ratings, ratings, "|".join(GAME_TYPES_BY_SHORT_FICS_NAME.keys())),
+            "(\d+) games displayed.")
 
         if self.helperconn.FatICS or self.helperconn.USCN:
             self.helperconn.expect_line(self.on_player_who, "%s(?:\s{2,}%s)+" %
@@ -48,17 +51,9 @@ class HelperManager(GObject.GObject):
         else:
             # New ivar pin
             # http://www.freechess.org/Help/HelpFiles/new_features.html
-            self.helperconn.expect_line(
-                self.on_player_whoI, "([A-Za-z]+)([\^~:\#. &])(\\d{2})" +
-                "(\d{1,4})([P E])" * 8 + "(\d{1,4})([PE]?)")
-            self.helperconn.expect_line(self.on_player_whoI_end,
-                                        "(\d+) Players Displayed.")
-            self.helperconn.expect_line(
-                self.on_player_connectI,
-                "<wa> ([A-Za-z]+)([\^~:\#. &])(\\d{2})" + "(\d{1,4})([P E])" *
-                8 + "(\d{1,4})([PE]?)")
-            self.helperconn.expect_line(self.on_player_disconnectI,
-                                        "<wd> ([A-Za-z]+)")
+            self.helperconn.expect_fromto(self.on_player_whoI, whoI, "(\d+) Players Displayed.")
+            self.helperconn.expect_line(self.on_player_connectI, "<wa> %s" % whoI)
+            self.helperconn.expect_line(self.on_player_disconnectI, "<wd> ([A-Za-z]+)")
 
         self.helperconn.expect_line(
             self.on_game_add,
@@ -75,7 +70,6 @@ class HelperManager(GObject.GObject):
             % (names, ratings, ratings, ratings, ratings, ratings),
             "is now available for matches.")
 
-        self.players = []
         # b: blitz      l: lightning   u: untimed      e: examined game
         # s: standard   w: wild        x: atomic       z: crazyhouse
         # B: Bughouse   L: losers      S: Suicide
@@ -84,56 +78,67 @@ class HelperManager(GObject.GObject):
         else:
             self.helperconn.client.run_command("who IbslwBzSLx")
 
-        self.games = []
         if self.helperconn.FatICS or self.helperconn.USCN:
             self.helperconn.client.run_command("games")
         else:
             self.helperconn.client.run_command("games /bslwBzSLx")
 
-    def on_game_list(self, match):
-        gameno, wrating, wname, brating, bname, private, shorttype, rated, min, \
-            inc, whour, wmin, wsec, bhour, bmin, bsec, wmat, bmat, color, movno = match.groups()
-        try:
-            gametype = GAME_TYPES_BY_SHORT_FICS_NAME[shorttype]
-        except KeyError:
-            return
+    def on_game_list(self, matchlist):
+        games = []
+        for match in matchlist[:-1]:
+            if isinstance(match, str):
+                if match:
+                    parts0, parts1 = match.split("[")
+                    gameno, wrating, wname, brating, bname = parts0.split()
+                    private = parts1[0]
+                    shorttype = parts1[1]
+                    rated = parts1[2]
+                    min = parts1[3:6]
+                    inc = parts1[7:10]
+                else:
+                    continue
+            else:
+                gameno, wrating, wname, brating, bname, private, shorttype, rated, min, \
+                    inc, whour, wmin, wsec, bhour, bmin, bsec, wmat, bmat, color, movno = match.groups()
+            try:
+                gametype = GAME_TYPES_BY_SHORT_FICS_NAME[shorttype]
+            except KeyError:
+                return
 
-        wplayer = self.connection.players.get(FICSPlayer(wname))
-        bplayer = self.connection.players.get(FICSPlayer(bname))
-        game = FICSGame(wplayer,
-                        bplayer,
-                        gameno=int(gameno),
-                        rated=(rated == "r"),
-                        private=(private == "p"),
-                        minutes=int(min),
-                        inc=int(inc),
-                        game_type=gametype)
+            wplayer = self.connection.players.get(wname)
+            bplayer = self.connection.players.get(bname)
+            game = FICSGame(wplayer,
+                            bplayer,
+                            gameno=int(gameno),
+                            rated=(rated == "r"),
+                            private=(private == "p"),
+                            minutes=int(min),
+                            inc=int(inc),
+                            game_type=gametype)
 
-        for player, rating in ((wplayer, wrating), (bplayer, brating)):
-            if player.status != IC_STATUS_PLAYING:
-                player.status = IC_STATUS_PLAYING
-            if player.game != game:
-                player.game = game
-            rating = self.parseRating(rating)
-            if gametype.rating_type in player.ratings and \
-                    player.ratings[gametype.rating_type].elo != rating:
-                player.ratings[gametype.rating_type].elo = rating
-
-        self.games.append(self.connection.games.get(game, emit=False))
+            for player, rating in ((wplayer, wrating), (bplayer, brating)):
+                if player.status != IC_STATUS_PLAYING:
+                    player.status = IC_STATUS_PLAYING
+                if player.game != game:
+                    player.game = game
+                rating = parseRating(rating)
+                if gametype.rating_type in player.ratings and \
+                        player.ratings[gametype.rating_type] != rating:
+                    player.ratings[gametype.rating_type] = rating
+                    player.emit("ratings_changed", gametype.rating_type, player)
+            game = self.connection.games.get(game, emit=False)
+            games.append(game)
+        self.connection.games.emit("FICSGameCreated", games)
+        # print(matchlist[-1].groups()[0], len(games))
 
     on_game_list.BLKCMD = BLKCMD_GAMES
 
-    def on_game_list_end(self, match):
-        self.connection.games.emit("FICSGameCreated", self.games)
-
-    on_game_list_end.BLKCMD = BLKCMD_GAMES
-
     def on_game_add(self, match):
         gameno, wname, bname, rated, game_type = match.groups()
         if game_type not in GAME_TYPES:
             return
-        wplayer = self.connection.players.get(FICSPlayer(wname))
-        bplayer = self.connection.players.get(FICSPlayer(bname))
+        wplayer = self.connection.players.get(wname)
+        bplayer = self.connection.players.get(bname)
         game = FICSGame(wplayer,
                         bplayer,
                         gameno=int(gameno),
@@ -155,21 +160,22 @@ class HelperManager(GObject.GObject):
             comment,
             wname=wname)
 
-        wplayer = FICSPlayer(wname)
         try:
-            wplayer = self.connection.players.get(wplayer, create=False)
-            wplayer.restore_previous_status(
-            )  # no status update will be sent by
+            wplayer = self.connection.players.get(wname)
+            wplayer.restore_previous_status()
+            # no status update will be sent by
             # FICS if the player doesn't become available, so we restore
             # previous status first (not necessarily true, but the best guess)
         except KeyError:
-            pass
-        bplayer = FICSPlayer(bname)
+            print("%s not in self.connections.players - creating" % wname)
+            wplayer = FICSPlayer(wname)
+
         try:
-            bplayer = self.connection.players.get(bplayer, create=False)
+            bplayer = self.connection.players.get(bname)
             bplayer.restore_previous_status()
         except KeyError:
-            pass
+            print("%s not in self.connections.players - creating" % bname)
+            bplayer = FICSPlayer(bname)
 
         game = FICSGame(wplayer,
                         bplayer,
@@ -185,12 +191,6 @@ class HelperManager(GObject.GObject):
         wplayer.game = None
         bplayer.game = None
 
-    @staticmethod
-    def parseRating(rating):
-        if rating[0] == " ":
-            rating = rating[1:]
-        return int(rating) if rating.isdigit() else 0
-
     @staticmethod
     def parseTitles(titles):
         _titles = set()
@@ -206,13 +206,13 @@ class HelperManager(GObject.GObject):
         name, status, titlehex, blitz, blitzdev, std, stddev, light, lightdev, \
             wild, wilddev, bughouse, bughousedev, crazyhouse, crazyhousedev, \
             suicide, suicidedev, losers, losersdev, atomic, atomicdev = match.groups()
-        player = self.connection.players.get(FICSPlayer(name))
-        self.players.append(player)
+        player = self.connection.players.get(name)
+
         titles = parse_title_hex(titlehex)
         if not player.titles >= titles:
             player.titles |= titles
 
-        for rtype, elo, dev in \
+        for rating_type, elo, dev in \
                 ((TYPE_BLITZ, blitz, blitzdev),
                  (TYPE_STANDARD, std, stddev),
                  (TYPE_LIGHTNING, light, lightdev),
@@ -222,10 +222,11 @@ class HelperManager(GObject.GObject):
                  (TYPE_BUGHOUSE, bughouse, bughousedev),
                  (TYPE_LOSERS, losers, losersdev),
                  (TYPE_SUICIDE, suicide, suicidedev)):
-            parse_rating = self.parseRating(elo)
-            if player.ratings[rtype].elo != parse_rating:
-                player.ratings[rtype].elo = parse_rating
-            player.ratings[rtype].deviation = DEVIATION[dev]
+            rating = parseRating(elo)
+            if player.ratings[rating_type] != rating:
+                player.ratings[rating_type] = rating
+                player.emit("ratings_changed", rating_type, player)
+            player.deviations[rating_type] = DEVIATION[dev]
 
         # do last so rating info is there when notifications are generated
         status = STATUS[status]
@@ -234,23 +235,31 @@ class HelperManager(GObject.GObject):
         if set_online and not player.online:
             player.online = True
 
+        return player
+
     def on_player_disconnectI(self, match):
         name = match.groups()[0]
-        self.connection.players.player_disconnected(FICSPlayer(name))
+        self.connection.players.player_disconnected(name)
 
-    def on_player_whoI(self, match):
-        self.on_player_connectI(match)
+    def on_player_whoI(self, matchlist):
+        players = []
+        for match in matchlist[:-1]:
+            if isinstance(match, str):
+                if match:
+                    players.append(self.on_player_connectI(re_whoI.match(match)))
+                else:
+                    continue
+            else:
+                players.append(self.on_player_connectI(match))
+
+        self.connection.players.emit("FICSPlayerEntered", players)
+        # print(matchlist[-1].groups()[0], len(players))
 
     on_player_whoI.BLKCMD = BLKCMD_WHO
 
-    def on_player_whoI_end(self, match):
-        self.connection.players.emit("FICSPlayerEntered", self.players)
-
-    on_player_whoI_end.BLKCMD = BLKCMD_WHO
-
     def on_player_who(self, match):
         for blitz, status, name, titles in whomatch_re.findall(match.string):
-            player = self.connection.players.get(FICSPlayer(name))
+            player = self.connection.players.get(name)
             if not player.online:
                 player.online = True
             status = STATUS[status]
@@ -259,23 +268,23 @@ class HelperManager(GObject.GObject):
             titles = self.parseTitles(titles)
             if not player.titles >= titles:
                 player.titles |= titles
-            blitz = self.parseRating(blitz)
-            if player.ratings[TYPE_BLITZ].elo != blitz:
-                player.ratings[TYPE_BLITZ].elo = blitz
+            blitz = parseRating(blitz)
+            if player.ratings[TYPE_BLITZ] != blitz:
+                player.ratings[TYPE_BLITZ] = blitz
+                player.emit("ratings_changed", TYPE_BLITZ, player)
 
     def on_player_connect(self, match):
         name = match.groups()[0]
-        player = self.connection.players.get(FICSPlayer(name))
-        self.players.append(player)
+        player = self.connection.players.get(name)
         player.online = True
 
     def on_player_disconnect(self, match):
         name = match.groups()[0]
-        self.connection.players.player_disconnected(FICSPlayer(name))
+        self.connection.players.player_disconnected(name)
 
     def on_player_unavailable(self, match):
         name, titles = match.groups()
-        player = self.connection.players.get(FICSPlayer(name))
+        player = self.connection.players.get(name)
         titles = self.parseTitles(titles)
         if not player.titles >= titles:
             player.titles |= titles
@@ -287,7 +296,7 @@ class HelperManager(GObject.GObject):
 
     def on_player_available(self, matches):
         name, titles, blitz, std, wild, light, bughouse = matches[0].groups()
-        player = self.connection.players.get(FICSPlayer(name))
+        player = self.connection.players.get(name)
 
         if not player.online:
             player.online = True
@@ -297,9 +306,10 @@ class HelperManager(GObject.GObject):
         if not player.titles >= titles:
             player.titles |= titles
 
-        for rtype, rating in ((TYPE_BLITZ, blitz), (TYPE_STANDARD, std),
+        for rating_type, rating in ((TYPE_BLITZ, blitz), (TYPE_STANDARD, std),
                               (TYPE_LIGHTNING, light), (TYPE_WILD, wild),
                               (TYPE_BUGHOUSE, bughouse)):
-            rating = self.parseRating(rating)
-            if player.ratings[rtype].elo != rating:
-                player.ratings[rtype].elo = rating
+            rating = parseRating(rating)
+            if player.ratings[rating_type] != rating:
+                player.ratings[rating_type] = rating
+                player.emit("ratings_changed", rating_type, player)
