@@ -5,8 +5,8 @@ from gi.repository import GObject
 from pychess.Utils.const import WHITE, FISCHERRANDOMCHESS, UNSUPPORTED
 from pychess.ic import BLKCMD_ASSESS, VariantGameType, DEVIATION, GAME_TYPES, \
     parse_title_hex, BLKCMD_UNSEEK, BLKCMD_SEEK, type_to_display_text, \
-    Variants
-from pychess.ic.FICSObjects import FICSPlayer, FICSSeek
+    Variants, RATING_TYPES
+from pychess.ic.FICSObjects import FICSSeek
 from pychess.System.Log import log
 
 rated = "(rated|unrated)"
@@ -115,7 +115,7 @@ class SeekManager(GObject.GObject):
 
         try:
             index = int(parts[0])
-            player = self.connection.players.get(FICSPlayer(seek['w']))
+            player = self.connection.players.get(seek['w'])
             player.titles |= parse_title_hex(seek['ti'])
             rated = seek['r'] == 'r'
             minutes = int(seek['t'])
@@ -156,13 +156,11 @@ class SeekManager(GObject.GObject):
                 return
         if gametype.variant_type in UNSUPPORTED:
             return
-        try:
-            ratingobj = player.ratings[gametype.rating_type]
-            if ratingobj.elo != rating:
-                ratingobj.elo = rating
-            ratingobj.deviation = deviation
-        except KeyError:
-            pass
+
+        if gametype.rating_type in RATING_TYPES and player.ratings[gametype.rating_type] != rating:
+            player.ratings[gametype.rating_type] = rating
+            player.deviations[gametype.rating_type] = deviation
+            player.emit("ratings_changed", gametype.rating_type, player)
 
         seek = FICSSeek(index,
                         player,
