@@ -23,7 +23,8 @@ from pychess.Players.engineNest import discoverer
 from pychess.Utils.const import HINT, SPY, SOUND_MUTE, SOUND_BEEP, SOUND_URI, SOUND_SELECT
 from pychess.Utils.IconLoader import load_icon, get_pixbuf
 from pychess.gfx import Pieces
-from .Background import hexcol
+from pychess.widgets import Background
+from pychess.widgets.Background import hexcol, newTheme
 
 firstRun = True
 
@@ -72,7 +73,8 @@ class GeneralTab:
         for key in ("firstName", "secondName", "showEmt", "showEval",
                     "autoPromote", "hideTabs", "closeAll", "faceToFace",
                     "showCords", "showCaptured", "figuresInNotation",
-                    "fullAnimation", "moveAnimation", "noAnimation"):
+                    "fullAnimation", "moveAnimation", "noAnimation",
+                    "showFICSgameno"):
             uistuff.keep(widgets[key], key)
 
         # Options on by default
@@ -102,9 +104,9 @@ def anal_combo_set_value(combobox, value, show_arrow_check, ana_check,
             index = 0
         combobox.set_active(index)
 
-    from pychess.Main import gameDic
+    from pychess.widgets.ionest import game_handler
     from pychess.widgets.gamewidget import widgets
-    for gmwidg in gameDic.keys():
+    for gmwidg in game_handler.gamewidgets:
         spectators = gmwidg.gamemodel.spectators
         md5 = engine.get('md5')
 
@@ -157,10 +159,10 @@ class HintTab:
         book_chooser_button.connect("file-set", select_new_book)
 
         def on_opening_check_toggled(check):
-            widgets["opening_hbox"].set_sensitive(check.get_active())
+            self.widgets["opening_hbox"].set_sensitive(check.get_active())
 
-        widgets["opening_check"].connect_after("toggled",
-                                               on_opening_check_toggled)
+        self.widgets["opening_check"].connect_after("toggled",
+                                                    on_opening_check_toggled)
 
         # Endgame
         default_path = os.path.join(getDataPrefix())
@@ -187,10 +189,10 @@ class HintTab:
         egtb_chooser_button.connect("current-folder-changed", select_egtb)
 
         def on_endgame_check_toggled(check):
-            widgets["endgame_hbox"].set_sensitive(check.get_active())
+            self.widgets["endgame_hbox"].set_sensitive(check.get_active())
 
-        widgets["endgame_check"].connect_after("toggled",
-                                               on_endgame_check_toggled)
+        self.widgets["endgame_check"].connect_after("toggled",
+                                                    on_endgame_check_toggled)
 
         # Analyzing engines
         from pychess.widgets import newGameDialog
@@ -216,54 +218,53 @@ class HintTab:
         conf.set("inv_ana_combobox", conf.get("inv_ana_combobox", default))
 
         def on_analyzer_check_toggled(check):
-            widgets["analyzers_vbox"].set_sensitive(check.get_active())
-            from pychess.Main import gameDic
-            if gameDic:
+            self.widgets["analyzers_vbox"].set_sensitive(check.get_active())
+            from pychess.widgets.ionest import game_handler
+            from pychess.widgets.gamewidget import widgets
+            if len(game_handler.gamewidgets) != 0:
                 if check.get_active():
-                    for gmwidg in gameDic.keys():
+                    for gmwidg in game_handler.gamewidgets:
                         gmwidg.gamemodel.restart_analyzer(HINT)
                         if not widgets["hint_mode"].get_active():
                             gmwidg.gamemodel.pause_analyzer(HINT)
                 else:
-                    for gmwidg in gameDic.keys():
+                    for gmwidg in game_handler.gamewidgets:
                         gmwidg.gamemodel.remove_analyzer(HINT)
 
-        widgets["analyzers_vbox"].set_sensitive(widgets[
+        self.widgets["analyzers_vbox"].set_sensitive(widgets[
             "analyzer_check"].get_active())
-        widgets["analyzer_check"].connect_after("toggled",
-                                                on_analyzer_check_toggled)
+        self.widgets["analyzer_check"].connect_after("toggled",
+                                                     on_analyzer_check_toggled)
 
         def on_invanalyzer_check_toggled(check):
-            widgets["inv_analyzers_vbox"].set_sensitive(check.get_active())
-            from pychess.Main import gameDic
-            if gameDic:
+            self.widgets["inv_analyzers_vbox"].set_sensitive(check.get_active())
+            from pychess.widgets.ionest import game_handler
+            if len(game_handler.gamewidgets) != 0:
                 if check.get_active():
-                    for gmwidg in gameDic.keys():
+                    for gmwidg in game_handler.gamewidgets:
                         gmwidg.gamemodel.restart_analyzer(SPY)
                         if not widgets["spy_mode"].get_active():
                             gmwidg.gamemodel.pause_analyzer(SPY)
                 else:
-                    for gmwidg in gameDic.keys():
+                    for gmwidg in game_handler.gamewidgets:
                         gmwidg.gamemodel.remove_analyzer(SPY)
 
-        widgets["inv_analyzers_vbox"].set_sensitive(widgets[
+        self.widgets["inv_analyzers_vbox"].set_sensitive(widgets[
             "inv_analyzer_check"].get_active())
-        widgets["inv_analyzer_check"].connect_after(
+        self.widgets["inv_analyzer_check"].connect_after(
             "toggled", on_invanalyzer_check_toggled)
 
         # Give widgets to keeper
 
         uistuff.keep(
-            widgets["ana_combobox"], "ana_combobox", anal_combo_get_value,
+            self.widgets["ana_combobox"], "ana_combobox", anal_combo_get_value,
             lambda combobox, value: anal_combo_set_value(combobox, value, "hint_mode", "analyzer_check", HINT))
         uistuff.keep(
-            widgets["inv_ana_combobox"], "inv_ana_combobox",
+            self.widgets["inv_ana_combobox"], "inv_ana_combobox",
             anal_combo_get_value,
             lambda combobox, value: anal_combo_set_value(combobox, value, "spy_mode", "inv_analyzer_check", SPY))
 
-        uistuff.keep(widgets["max_analysis_spin"],
-                     "max_analysis_spin",
-                     first_value=3)
+        uistuff.keep(self.widgets["max_analysis_spin"], "max_analysis_spin", first_value=3)
 
 # Sound initing
 
@@ -588,6 +589,43 @@ class ThemeTab:
         sets and board colours
     """
     def __init__(self, widgets):
+        self.widgets = widgets
+
+        # Background image
+        path = conf.get("welcome_image", addDataPrefix("glade/clear.png"))
+        conf.set("welcome_image", path)
+
+        image_chooser_dialog = Gtk.FileChooserDialog(
+            _("Select background image file"), None, Gtk.FileChooserAction.OPEN,
+            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
+             Gtk.ResponseType.OK))
+        image_chooser_button = Gtk.FileChooserButton.new_with_dialog(
+            image_chooser_dialog)
+
+        filter = Gtk.FileFilter()
+        filter.set_name(_("Images"))
+        filter.add_pattern("*.bmp")
+        filter.add_pattern("*.jpg")
+        filter.add_pattern("*.png")
+        filter.add_pattern("*.svg")
+        image_chooser_dialog.add_filter(filter)
+        image_chooser_button.set_filename(path)
+
+        self.widgets["imageChooserDock"].add(image_chooser_button)
+        image_chooser_button.show()
+
+        def select_new_image(button):
+            new_image = image_chooser_dialog.get_filename()
+            if new_image:
+                conf.set("welcome_image", new_image)
+                from pychess.widgets.TaskerManager import tasker
+                newTheme(tasker, background=new_image)
+                tasker.queue_draw()
+            else:
+                # restore the original
+                image_chooser_dialog.set_filename(path)
+
+        image_chooser_button.connect("file-set", select_new_image)
 
         # Board Colours
 
